#!/bin/sh
#\
exec stash $0 $@

# last edition 07-01-2005

load_package gap
source /my/modules/directory/GenALA/bin/gap4.inc

# initialise C and Tcl tag id and tag type arrays
InitTagArray         ;# if work includes usage of tag information
# InitCanvasConstants  ;# canvas!
InitLists            ;# initialize built-in lists
# InitListTrace        ;# display of lists

# turn off the annoying bell
error_bell 0


set re_slc_read ""

# options from command line arguments
while {$argc > 0 && "[string index [lindex $argv 0] 0]" == "-"} {
  set arg [lindex $argv 0]
  set argv [lrange $argv 1 $argc] ; incr argc -1 ;

  # stop scanning for options
  if {$arg == "--"} {
    break

  # select readings by regular expression
  } elseif {$arg == "-r"} {
    set re_slc_read "[lindex $argv 0]"
    set argv [lrange $argv 1 $argc] ; incr argc -1 ;

  # usage help
  } elseif {$arg == "-h"} {
    set argc 0
    break

  # xyz: non-argument option
  } elseif {$arg == "-xyz"} {
    set FlagStripPads 1

  # unknown option
  } else {
    puts stderr "ERROR: invalid option '$arg'"
    exit 1
  }
}

# parameters from command line arguments
if {$argc < 2} {
  puts stderr ""
  puts stderr "USAGE"
  puts stderr "exportpreass.tcl  \[-h\] \[-r S\]  GapDB.V ExportFolder \[ReadID(s)\]"
  puts stderr ""
  puts stderr "ARGUMENTS"
  puts stderr "GapDB.V       file path of GAP4 database"
  puts stderr "ExportFolder  folder for export output"
  puts stderr "ReadID        optional: read identifiers (numerical or alphanumerical),"
  puts stderr "              default: all"
  puts stderr ""
  puts stderr "OPTIONS"
  puts stderr "-h    show this command line usage help"
  puts stderr "-r S  select reads by applying regexp. Do not default to all readings if"
  puts stderr "      there is no match."
  puts stderr ""
  exit 1
}
set DbSpec    [lindex $argv 0]
set expfolder [lindex $argv 1]
  # the folder needn't to be cleaned
  # actually, it mustn't be cleaned, cmp. Gap.pl -TransposExp
set readings  [lrange $argv 2 [expr $argc-1]]
set readings_llen [llength $readings]


################################################################################
# work
################################################################################

# open the database
set io [OpenDatabaseR $DbSpec]

# select readings by regular expression
if {$re_slc_read != ""} {
  foreach ReadEntry [ListGet "allreadings"] {
    if {[regexp $re_slc_read $ReadEntry]} {
      lappend readings $ReadEntry
    }
  }
  if {[llength $readings] == $readings_llen} {
    puts stderr "WARNING: no match to regexp '$re_slc_read'"
  }
} else {
# default reading list
  if {$readings == "" || $readings == {}} {
    set readings [ListGet "allreadings"]
  }
}

# export readings
extract_readings         \
  -io $io                \
  -readings $readings    \
  -directory $expfolder  \
  -format    1

# close the database
close_db -io $io
exit
