#!/bin/sh
#\
exec stash $0 $@

# ======================================================== #
#
# 3.2.2005, FLI, Jena, Old Europe
# by Markus Schilhabel
# 
# Allows list if contigs -C
#
# ======================================================== #

set VERSION 1.8
set DATUM "12.6.2006"
set AUTHOR "mbs@fli-leibniz.de"
set ThisPrg [lindex [split $argv0 /] end]

set LibPath "/my/modules/directory/GenALA/lib"

source $LibPath/StdTcl.tcm
array set LastModuleInfos [ModuleInfo]
VersControl $LastModuleInfos(Name) $LastModuleInfos(Version) 1.1

# define options
set OptList {t g c r l h a v d C}
set MustBeOpt {g}

# Optionen mit Wert
set NeedsVal {g a t C}

# define defaults
#set DefaultValue(a) 1;  # Limit
set DefaultValue(t) ""; # Tags
set DefaultValue(l) 0;  # not looking

#load the necessities
load_package gap


# ======================================================== #
# =======================  USAGE  ======================== #
# ======================================================== #

proc PrintUsage { } {
   global ThisPrg
   puts stderr ""
   puts stderr "* ----------------------------------------------------------------"
   puts stderr "*"
   puts stderr "* HELP:"
   puts stderr "*  This program is designed to remove tags from an existing gap"
   puts stderr "*  project."
   puts stderr "*"
   puts stderr "*  Using the -t option restricts to the named tags, the -c and"
   puts stderr "*  -r options direct the deletion to read- or contig-tags."
   puts stderr "*"
   puts stderr "* EXAMPLE:"
   puts stderr "*  Remove all tRNA and rRNA tags from the contig longer than 800"
   puts stderr "*   of test.0:   $ThisPrg -g test.0 -c -t TRNA,RRNA -a 800"
   puts stderr "*"
   puts stderr "* ----------------------------------------------------------------"
   puts stderr ""
   return
}

# ======================================================== #
# =======================  HELP  ========================= #
# ======================================================== #

proc PrintHelp { } {
   global ThisPrg VERSION DATUM AUTHOR
   puts stderr ""
   puts stderr "(deletes cons or read tags) USAGE:"
   puts stderr "$ThisPrg -g gap_project.v -r -c \[or -C <NAME>\] \[-l -a <NUM> -t <TAG1>,<TAG2>,... or all tags\]"
   puts stderr ""
   puts stderr "  -g FILE.V   name of gap_project"
   puts stderr "  -r          delete tags from reads"
   puts stderr "  -c          delete tags from consensus (all)"
   puts stderr "     or"
   puts stderr "  -C NAME     delete tags from listed consensus (comma seperated, no blanks)"
   puts stderr " (-a NUM      delete above contig length (useless for read tags!))"
   puts stderr " (-l          only looking, not doing anything yet)"
   puts stderr " (-t TAGS     TAG1,TAG2,TAG3, ... name of tags to be deleted,"
   puts stderr "              comma seperated, no blanks)"
   puts stderr " (-v          be verbose)"
   puts stderr "  -h          print a help"
   puts stderr ""
   puts stderr "        === DATE: $DATUM,  VERSION: $VERSION,  AUTHOR: $AUTHOR ==="
   return
}

# ======================================================== #
# ====================  check args  ====================== #
# ======================================================== #

if {$argc < 1} {
   #puts "argv0: $argv0, argc: $argc"
   PrintHelp
   exit
}

# ======  Load Opt Module =====
source $LibPath/GetOpts.tcm
array set LastModuleInfos [ModuleInfo]
VersControl $LastModuleInfos(Name) $LastModuleInfos(Version) 1.2


if {[array exists DefaultValue] == 1} {
   set ArgList [GetOpts $OptList $NeedsVal [array get DefaultValue] ]
} else {
   set ArgList [GetOpts $OptList $NeedsVal ""]
}

if {$ArgList == 0 } {
   PrintHelp
   exit
} else {
   array set OptHash $ArgList
}

# ===  HELP  ====
if  {[info exists OptHash(h)]} {
   PrintUsage
   PrintHelp
   exit
}

# Alle MustBeOpt da?
foreach Must $MustBeOpt {
   if {![info exists OptHash($Must)]} {
      puts "need option $Must"
      PrintHelp;  #  $DefaultValue(f) $DefaultValue(o) $DefaultValue(s)
      exit
   }
}

# -c und -C ?? Nur eins!
if {[info exists OptHash(C)] && [info exists OptHash(c)]} {
   PrintHelp
   puts stderr ""
   puts stderr "ERROR: -c OR -C"
   exit
}

# weder -r noch (-c oder -C)
if {![info exists OptHash(r)] && !([info exists OptHash(C)] || [info exists OptHash(c)])} {
   PrintHelp
   puts stderr ""
   puts stderr "ERROR: -r OR (-c OR -C)"
   exit
}


# ====   parse Opts  ========
set verbose 0
set debug 0
set offReads 0
set offContigs 0
set LowLimit 1
set Tags ""
foreach key [array names OptHash] {
   # if -v set verbosity
   if {$key == "v"} {
      set verbose 1
   } elseif {$key == "r"} {
      # it -r set reads
      set offReads 1
   } elseif {$key == "c"} {
      set offContigs 1
      set OnlyContigs {all}
   } elseif {$key == "C"} {
      set offContigs 1
      set OnlyContigs [split $OptHash(C) ,]
   } elseif {$key == "a"} {
      set LowLimit $OptHash(a)
   } elseif {$key == "t"} {
      set Tags [split $OptHash(t) ,]
   } elseif {$key == "d"} {
      set debug 1
   }
}

if {$debug > 0} {
   puts "OptHash: "
   puts [array names OptHash]
   puts "OnlyContigs: $OnlyContigs"
}


# ======================================================== #
# ===================  Program code  ===================== #
# ======================================================== #

# load Modules
source $LibPath/GapTcl.tcm
array set LastModuleInfos [ModuleInfo]
VersControl $LastModuleInfos(Name) $LastModuleInfos(Version) 1.3

# parse Project Name
set GapN [GetGapName $OptHash(g)]
#puts "GapN: $GapN, db: [lindex $GapN 0], v: [lindex $GapN 1]"

if {[file readable [join $GapN .]] && [file readable "[join $GapN .].aux" ]} {
   if {$OptHash(l) == ""} {
      puts stderr "... only looking"
      set io [open_db -name [lindex $GapN 0] -version [lindex $GapN 1] -access r]
   } else {
      set io [open_db -name [lindex $GapN 0] -version [lindex $GapN 1] -access rw]
   }
} else {
   puts stderr "Can't access $OptHash(g)"
   exit
}

# ======================================================== #
# =========================  STASH  ====================== #
# ======================================================== #

set db [io_read_database $io]
set alist {}
set ContigMatches 0
set ReadMatches 0

#  ##################################################  #
#                        CONTIGS
#  ##################################################  #
if {$offContigs == 1} {
   # From contigs:
   set nc [keylget db num_contigs]

   for {set cnum 1} {$cnum <= $nc} {incr cnum} {
      set contig [io_read_contig $io $cnum]
      
      # check if this contig's wanted
      set ContigName [io_read_reading_name $io [keylget contig left]]
      if {[lsearch -exact $OnlyContigs $ContigName] >= 0 || [lsearch -exact $OnlyContigs "all"] >= 0 } {
         if {$verbose > 0} {
            # limit?
            puts "Len: [keylget contig length]"
            puts "ContigName: $ContigName"
         }

         if {[keylget contig length] >= $LowLimit} {
            for {set anno [keylget contig annotations]} {$anno} {set anno [keylget a next]} {
               set a [io_read_annotation $io $anno]
               # selected tags given?
               if {$Tags != ""} {
                  #puts "Removing $Tags"
                  # search selected tag
                  foreach type $Tags {
                     #puts "Removing $type tags of contig $cnum"
                     if {[keylget a type] == $type} {
                        set ContigMatches [ expr $ContigMatches + 1 ]
                        if {$verbose == 1} {
                           puts "Removing [keylget a type] tag of contig $ContigName"
                        }
                        lappend alist $anno [keylget a type] \
                                            [keylget a position] \
                                            [keylget a length] \
                                            [keylget a strand]
                     }
                  }
               } else {
                  #puts "Removing all cons tags of contig $cnum"
                  set ContigMatches [ expr $ContigMatches + 1 ]
                  if {$verbose == 1} {
                     puts "Removing [keylget a type] tag of contig $ContigName"
                  }
                  lappend alist $anno [keylget a type] \
                                      [keylget a position] \
                                      [keylget a length] \
                                      [keylget a strand]
               }
            }
         }
      }
   }
}

#  ##################################################  #
#                        READS
#  ##################################################  #

if {$offReads == 1} {
   # from reads
   set nr [keylget db num_readings]
   for {set rnum 1} {$rnum <= $nr} {incr rnum} {
      set r [io_read_reading $io $rnum]
      for {set anno [keylget r annotations]} {$anno} {set anno [keylget a next]} {
         set a [io_read_annotation $io $anno]
         # selected tags given?
         if {$Tags != ""} {
            foreach type $Tags {
               if {[keylget a type] == $type} {
                  set ReadMatches [ expr $ReadMatches + 1 ]
                  if {$verbose == 1} {
                     puts "Removing [keylget a type] tag of read $rnum"
                  }
                  lappend alist $anno [keylget a type] \
                                      [keylget a position] \
                                      [keylget a length] \
                                      [keylget a strand]
               }
            }
         } else {
            set ReadMatches [ expr $ReadMatches + 1 ]
            if {$verbose == 1} {
               puts "Removing [keylget a type] tag of read $rnum"
            }
            lappend alist $anno [keylget a type] \
                                [keylget a position] \
                                [keylget a length] \
                                [keylget a strand]
         }
      }
   }
}


if {$OptHash(l) == ""} {
   puts "will remove $ContigMatches contig tags"
   puts "will remove $ReadMatches read tags"
   puts "nothing happend!"
} else {
   # Now remove this list
   puts "removing $ContigMatches contig tags"
   puts "removing $ReadMatches read tags"
   delete_anno_list -io $io -annos $alist
}

close_db -io $io

exit
