#!/bin/sh
#\
exec stash $0 $@

# ======================================================== #
#
# 2.2.2005, FLI, Jena, Germany, Old Europe
# by Markus Schilhabel
# 
# ======================================================== #
# 
# exportiert gap Sequenzen
# 
# ======================================================== #


load_package gap

set VERSION 1.51
set DATUM "19.6.2006"
set AUTHOR "mbs@fli-leibniz.de"
set ThisPrg [lindex [split $argv0 /] end]

set LibPath "/my/modules/directory/GenALA/lib"

source $LibPath/StdTcl.tcm
array set LastModuleInfos [ModuleInfo]
VersControl $LastModuleInfos(Name) $LastModuleInfos(Version) 1.0

# define options
set OptList {g h f o s c v}
set MustBeOpt {g f o s}
# Optionen mit Wert
set NeedsVal {g f o s c}

# define defaults
set DefaultValue(o) "cons"
set DefaultValue(f) "F"
set DefaultValue(s) "n"
#set output_format 2
set upperL 0
set lowerL 1

# ======================================================== #
# =======================  USAGE  ======================== #
# ======================================================== #

proc PrintUsage { } {
   global ThisPrg
   puts stderr ""
   puts stderr "* --------------------------------------------------------------"
   puts stderr "*                                                               "
   puts stderr "* HELP:                                                         "
   puts stderr "*  exports gap sequences                                        "
   puts stderr "*                                                               "
   puts stderr "* EXAMPLE:                                                      "
   puts stderr "*  Save the fasta sequence of a GAP4 project to cons            "
   puts stderr "*   $ThisPrg -g test.0                                          "
   puts stderr "*                                                               "
   puts stderr "*  Save an experiment file of a GAP4 project to cons.exp        "
   puts stderr "*   $ThisPrg -g test.0 -f X -o cons.exp                         "
   puts stderr "*                                                               "
   puts stderr "* --------------------------------------------------------------"
   puts stderr ""
   return
}

# ======================================================== #
# =======================  HELP  ========================= #
# ======================================================== #

proc PrintHelp { } {
   global ThisPrg VERSION DATUM AUTHOR DefaultValue
   puts stderr ""
   puts stderr "(saves consensus like GAP4) USAGE:"
   puts stderr "$ThisPrg -g GAP_PROJECT \[ -c NAME -f \[F\|X\|S\] -o TXT -s \[y\|n\]\]"
   puts stderr ""
   puts stderr "  -g FILE     Gap4 project name"
   puts stderr "  -c NAME(S)  selected contigs in C1,C2,C3    (default: all)"
   puts stderr "  -f FORMAT   X(periment), F(asta), S(taden)  (default: $DefaultValue(f))"
   puts stderr "  -o FILE     output filename                 (default: $DefaultValue(o))"
   puts stderr "  -s y/n      Strip pads \[y or n\]             (default: $DefaultValue(s))"
#   puts stderr "  -u NUM      upper size-limit                (default: $upperL)"
   puts stderr "  -h          print a help"
   puts stderr ""
   puts stderr "        === DATE: $DATUM,  VERSION: $VERSION,  AUTHOR: $AUTHOR ==="
   return
}


# ======================================================== #
# ====================  check args  ====================== #
# ======================================================== #

# too few arguments
if {$argc < 1} {
   PrintHelp
   exit
}

source $LibPath/GetOpts.tcm
array set LastModuleInfos [ModuleInfo]
VersControl $LastModuleInfos(Name) $LastModuleInfos(Version) 1.2

# Has default values?
if {[array exists DefaultValue] == 1} {
   set ArgList [GetOpts $OptList $NeedsVal [array get DefaultValue] ]
} else {
   set ArgList [GetOpts $OptList $NeedsVal ""]
}

if {$ArgList == 0 } {
   PrintHelp
   exit
} else {
   array set OptHash $ArgList
}


# ===  HELP  ====
if  {[info exists OptHash(h)]} {
   PrintUsage
   PrintHelp
   exit
}

# Alle MustBeOpt da?
foreach Must $MustBeOpt {
   
   if {![info exists OptHash($Must)]} {
      puts "need option $Must"
      PrintHelp
      exit
   }
}


# ======================================================== #
# ===================  Program code  ===================== #
# ======================================================== #

source $LibPath/GapTcl.tcm
array set LastModuleInfos [ModuleInfo]
VersControl $LastModuleInfos(Name) $LastModuleInfos(Version) 1.2


set GapN [GetGapName $OptHash(g)]

if {[file readable [join $GapN .]] && [file readable "[join $GapN .].aux"]} {
   set io [open_db -name [lindex $GapN 0] -version [lindex $GapN 1] -access r]
} else {
   puts stderr "Can't access $OptHash(g)"
   exit
}

# -s stripping pads
if {$OptHash(s) == "y"} {
   set StripPads 1
} elseif {$OptHash(s) == "n"} {
   set StripPads 0
} else {
   puts stderr " Unknown value \"$OptHash(s)\" using n"
   set StripPads 0
}

# -f save format staden = 1, fasta = 2, experiment = 3
if {$OptHash(f) == "X"} {
   set output_format 3
} elseif {$OptHash(f) == "F"} {
   set output_format 2
} elseif {$OptHash(f) == "S"} {
   set output_format 1
} else {
   puts stderr " Unknown format \"$OptHash(f)\" using F"
   set output_format 2
}

# selected contigs
if {[info exists OptHash(c)]} {
   set CtgSel [split $OptHash(c)]
   foreach ctg $CtgSel {
      set CTG($ctg) 1
   }
}


# ======================================================== #
# =======================  STASH  ======================== #
# ======================================================== #

# We need to adjust maxseq. Otherwise, function get_consensus won't work
#   properly.
set maxseq [expr round ([db_info t_contig_length $io] * 1.2) ]

#puts "ContigsIn: $ContigsIn"

if  {[info exists OptHash(c)]} {
   #puts "Opt c: $OptHash(c)"
   set OnlyContigs [split $OptHash(c) ,]
   set ContigsIn [CreateAllContigList $io]
   foreach ContigName $ContigsIn {
      if {[lsearch -exact $OnlyContigs $ContigName] >= 0} {
         #puts "index:  [lsearch -exact $OnlyContigs $ContigName]"
         lappend ContigList $ContigName
      }
   }
} else {
   set ContigList [CreateAllContigList $io]
}

#puts "ContigList: $ContigList"

get_consensus -io $io            \
 -outfile       $OptHash(o)      \
 -contigs       $ContigList      \
 -format        $output_format   \
 -annotations   1                \
 -truncate      1                \
 -strip_pads    $StripPads

close_db -io $io

# ======================================================== #
# ====================  END of Prog  ===================== #
# ======================================================== #
exit
