#!/bin/sh
#\
exec stash $0 $@

################################################################################
#
#  Tcl Script to Remove/Disassemble Readings from/in a GAP4 Database
#
#  all rights reserved
#  Karol Szafranski at IMB Jena, Dept. Genome Analysis, 1999-2001
#  szafrans@imb-jena.de
#
################################################################################


################################################################################
# intro
################################################################################

load_package gap
source /my/modules/directory/GenALA/bin/gap4.inc

# initialise C and Tcl tag id and tag type arrays
# InitTagArray         ;# if work includes usage of tag information
# InitCanvasConstants  ;# canvas!
# InitLists            ;# initialize built-in lists
# InitListTrace        ;# display of lists

# turn off the annoying bell
error_bell 0

# output directives (may be changed by command line options)
set FlagRemove 0


# options from command line arguments
while {$argc > 0 && "[string index [lindex $argv 0] 0]" == "-"} {
  set arg [lindex $argv 0]
  set argv [lrange $argv 1 $argc] ; incr argc -1 ;

  # stop scanning for options
  if {$arg == "--"} {
    break

  # usage help
  } elseif {$arg == "-h"} {
    set argc 0
    break

  # strip pads
  } elseif {$arg == "-r"} {
    set FlagRemove 0

  # unknown option
  } else {
    puts stderr "ERROR: invalid option '$arg'"
    exit 1
  }
}

# parameters from command line arguments
if {$argc != 2} {
  puts stderr ""
  puts stderr "USAGE"
  puts stderr "readdisassemble.tcl  \[-h\] \[-r\]  GapDB.V FofnReads"
  puts stderr ""
  puts stderr "ARGUMENTS"
  puts stderr "GapDB.V    file path of GAP4 database"
  puts stderr "FofnReads  file of read identifiers"
  puts stderr ""
  puts stderr "OPTIONS"
  puts stderr "-h    show this command line usage help"
  puts stderr "-r    remove reads from database (default: don't)"
  puts stderr ""
  exit 1
}
set DbSpec  [lindex $argv 0]
set fofn    [lindex $argv 1]
if {! [file exists $fofn]} {
  puts stderr "ERROR: unable to find file of filenames: $fofn"
  exit 1
}


################################################################################
# work
################################################################################

# read fofn
set ListRead [exec cat $fofn]

# open the database, check for errors
set io [OpenDatabaseRW $DbSpec]
capture {check_database -io $io} ErrorBefore
set ErrorBeforeNum 0
while {[regsub "\n" "$ErrorBefore" a ErrorBefore]} {
  incr ErrorBeforeNum
}
set ErrorBeforeNum [expr $ErrorBeforeNum - 1]


### disassemble readings staden-1-5-3
#disassemble_readings \
#  -io $io \
#  -readings $ListRead \
#  -move $FlagRemove

### Disassembly (older staden packages)
disassemble_readings \
  -io $io \
  -readings $ListRead \
  -all 1 \
  -remove 1




# check for errors, close the database
capture {puts -nonewline [format "\n%[string length $ErrorBefore]s" ""]} ErrorAfter
capture {check_database -io $io} ErrorAfter
set ErrorAfterNum 0
while {[regsub "\n" "$ErrorAfter" a ErrorAfter]} {
  incr ErrorAfterNum
}
set ErrorAfterNum [expr $ErrorAfterNum - 1]
close_db -io $io
if {$ErrorAfterNum > $ErrorBeforeNum} {
#  exec mv -f $DbPathName.X     $DbSpec
#  exec mv -f $DbPathName.X.aux $DbSpec.aux
#  exec mv -f $DbPathName.X.log $DbSpec.log
  puts stderr "ERROR: found (additional) database errors following the procedure"
#  puts "GAP project left unchanged"
  exit 1
} else {
#  exec rm -f $DbPathName.X    
#  exec rm -f $DbPathName.X.aux
#  exec rm -f $DbPathName.X.log
  exit 0
}
