package CloneLimit;

use strict;

use vars qw (@EXPORT @ISA);
use Exporter;
use POSIX;
@ISA    = qw (Exporter);
@EXPORT = qw (&calculate_clonelimit);


sub calculate_clonelimit {
	my( $pRead, $pContig, $pClone, $pEndread ) = @_;
	my( $item, @reads, @contigs, $read, $run, %bank, $bank, @sorted, $average );
	my( $median, $mean, $deviation, @limit, $limit, $i, $dezil, $endposition );

	%bank = ();

	print "\n\n\t\tcalculation of clone limits\n\n";

	foreach $item ( keys %$pClone ){

		( $bank = $item )   =~ s/(\S+[a-z])\d+[a-z]\d{2}/$1/;
		$bank{$bank}{limit} = 0;				# initialisation for each kind of clone names

		if( $#{$$pClone{$item}{runs}} > 0 ){
			@reads   = ();
			@contigs = ();
		

#-------------------- are all runs in one contig? ------------------------------

			foreach $run ( @{$$pClone{$item}{runs}} ){
				$read = join '.', $item, $run;
				push( @reads, $read );

				if( @contigs == 0 ){
					push @contigs, $$pRead{$read}{contig}{id};
				} elsif ( $$pRead{$read}{contig}{id} !~ /$contigs[$#contigs]/ ){
					push @contigs, $$pRead{$read}{contig}{id};
				} else {}
			}
			
#-------------------------------------------------------------------------------
			if( $#contigs == 0 ){

				if(( $$pRead{$reads[0]}{length} > 0 && $$pRead{$reads[$#reads]}{length} < 0 ) || \
				  (( $$pRead{$reads[0]}{length} < 0 && $$pRead{$reads[$#reads]}{length} > 0 ) && \
				     $$pRead{$reads[$#reads]}{position} < ( $$pRead{$reads[0]}{position} - $$pRead{$reads[0]}{length}))){
					 
					$endposition = 0;
					
					foreach $read ( @reads ){
						if(( $$pRead{$read}{position} + abs( $$pRead{$read}{length} )) > $endposition ){
							$endposition = $$pRead{$read}{position} + abs( $$pRead{$read}{length} );
						}
					} 

					$$pClone{$item}{length} = $endposition - $$pRead{$reads[0]}{position};
					$bank{$bank}{sum}      += $$pClone{$item}{length};
					$bank{$bank}{sqrsum}   += $$pClone{$item}{length}**2;
					$bank{$bank}{num}++;
					push @{$bank{$bank}{stack_length}}, $$pClone{$item}{length};
				}
			} else {
				$$pClone{$item}{status} = 0;
			}
		}
	}

#-------------------------------------------------------------------------------
#------------------------------ Statistics -------------------------------------

	foreach $item ( sort keys %bank ){
		@limit  = ();
		@sorted = ();

#------------------- Calculation of average and deviation ----------------------

		if( defined $bank{$item}{num} ){
			if( $bank{$item}{num} > 9 ){
			$average = $bank{$item}{sum} / $bank{$item}{num};

			if ( $bank{$item}{num} > 1 ){
				$deviation = ( ( $bank{$item}{num} * $bank{$item}{sqrsum} - $bank{$item}{sum}**2) / ( $bank{$item}{num} * ( $bank{$item}{num} - 1 )) )**0.5;
			} else {
				$deviation = $bank{$item}{sum}; 
			}

			push @limit, ( ceil(   2.2 * $average				/ 100 ) * 100 );
			push @limit, ( ceil( ( 2   * $deviation + $average) / 100 ) * 100 );
			@sorted = sort { $a <=> $b } @limit;

			$bank{$item}{limit}     = $sorted[1];
			$bank{$item}{average}   = sprintf "%.0f", $average;
			$bank{$item}{deviation} = sprintf "%.0f", $deviation;

#-------------------------- Calculation of deciles ------------------------------

			@sorted = ();
			@sorted = sort { $a <=> $b } @{$bank{$item}{stack_length}};

			while( $sorted[$#sorted] > $bank{$item}{limit} ){
				pop @sorted;
			}

			for( $i = 1; $i < 10; $i++ ){
				$dezil = sprintf "%.0f", (( $#sorted + 1) / 10 * $i );
				push @{$bank{$item}{dezil}}, $sorted[$dezil];
			}


			if( $#sorted % 2 ){
				$median = $sorted[$#sorted/2];
			} else {
				$mean = ($#sorted - 1) / 2;
				$median = ( $sorted[$mean] + $sorted[$mean+1] ) / 2;
			}

			$bank{$item}{median} = sprintf "%.0f", $median;
#-------------------------- Recalculation of clone limit -----------------------
                        print "$item\n";
			$bank{$item}{limit}  = $bank{$item}{dezil}[8] + ( $bank{$item}{dezil}[8] - $bank{$item}{dezil}[7] ) * 4;
			
			delete($bank{$item}{stack_length});
			delete($bank{$item}{sum});
			delete($bank{$item}{sqrsum});
			}
		}
	}
	
#-------------------------------------------------------------------------------
#-------------------------- Evaluation of Status  ------------------------------
	foreach $item ( keys %$pClone ){
		( $bank = $item ) =~ s/(\S+[a-z])\d+[a-z]\d{2}/$1/;
		

		if( $$pClone{$item}{length} > $bank{$bank}{limit} ){
			$$pClone{$item}{status} = -1;
		} else {
			$$pClone{$item}{status} = 1 if( $$pClone{$item}{length} > 0 );
		}
	}
	return( \%bank );
}

1;
