package EndReads;

use strict;

use vars qw (@EXPORT @ISA);
use Exporter;
@ISA    = qw (Exporter);
@EXPORT = qw ( &endreads &calculate_likelihood );





################################################################################
#
#   NOTE:
#   1. look for all clone banks with length > 0.
#   2. look for all clones with 'not unsure' status (-1)
#   3. exists more than one run
#   4. are reads in the terminal contig regions? 

# we need the information:
# 		clone-name
# 		exists read pairs: 0 or 1
# 		read orientation:  1 or -1
# 		probability:	   0.0 - 1.0


# ToDo: split this function in some subfunctions; it's too complex

sub endreads {
	my( $pRead, $pContig, $pClone, $pEndread, $pBank, $opt ) = @_;
	my( $item, $bank, $clone, $probability, $read, $distance );
	my( $low_limit, $high_limit, $run, @contigs, @reads, %distance );
	my( $orient, @sub_reads, $contig, %reads, $contig_id, %bridge_clones );


	%bridge_clones = ();
	if( $opt ){
	foreach $item ( keys %$pClone ){

		( $bank = $item ) =~ s/(\S+[a-z])\d+[a-z]\d{2}/$1/;

#------------------- only clones of a bank with known limit --------------------
		if( $$pBank{$bank}{limit} > 0 ){

#---------------------------- only good clones ---------------------------------
			if( $$pClone{$item}{status} > -1 ){
#-------------- there's existing more than one read of this clone --------------
				if( $#{$$pClone{$item}{runs}} > 0 ){
					@contigs = ();
					@reads   = ();
					
#------------------------ are all runs in one contig? --------------------------
					foreach $run ( @{$$pClone{$item}{runs}} ){
						$read = join '.', $item, $run;
						push @reads, $read;
						
						if( @contigs == 0 ){
							push @contigs, $$pRead{$read}{contig}{id};
						} elsif ( $$pRead{$read}{contig}{id} !~ /$contigs[$#contigs]/ ){
							push @contigs, $$pRead{$read}{contig}{id};
						} else {}
					}
					
#------------------------- all runs are in one contig! -------------------------
#----------------- only + or - runs of one clone in one contig! ----------------
					if( $#contigs == 0 && $$pClone{$item}{length} != 0){
						$distance = 0;
						$orient   = 2;

#--------------------- have all reads the same orientation? --------------------
						foreach $read ( @reads ){
							if( $orient != 0 ){
								if( $$pRead{$read}{length} < 0 ){
									if( $orient == -1 || $orient == 2 ){
										$orient = -1;
									} else {
										$orient = 0;
									}
								}
								if( $$pRead{$read}{length} > 0 ){
									if( $orient == 1 || $orient == 2 ){
										$orient = 1;
									} else {
										$orient = 0;
									}
								}
							}
						}
#--------------------- all reads have the same orientation! --------------------
						if( $orient != 0 ){

							if( $orient < 0 ){
							
								if( $distance < ( $$pRead{$read}{position} + abs( $$pRead{$read}{length} ) - 1 )){
									$distance = $$pRead{$read}{position} + abs( $$pRead{$read}{length} ) - 1;
								}

								if( $distance < $$pBank{$bank}{limit} ){
									$probability = &calculate_likelihood( $distance, $pBank, $bank);

									if( $probability > 0.2 ){
										$contig_id = $$pRead{$read}{contig}{id};
									
										$$pContig{$contig_id}{extendclone}{$item}{id}       = $item;
										$$pContig{$contig_id}{extendclone}{$item}{complete} = 1;
										$$pContig{$contig_id}{extendclone}{$item}{orient}   = $orient;
										$$pContig{$contig_id}{extendclone}{$item}{prob}     = $probability;

										$bridge_clones{$item} = 1;						
									}
								}
							} else {

								foreach $read ( @reads ){

									if( $distance < ( $$pRead{$read}{contig}{length} - $$pRead{$read}{position} + 1 )){
										$distance = $$pRead{$read}{contig}{length} - $$pRead{$read}{position} + 1;
									}
								}
						
								if( $distance < $$pBank{$bank}{limit} ){
									$probability = &calculate_likelihood( $distance, $pBank, $bank);

									if( $probability > 0.2 ){
										$contig_id = $$pRead{$read}{contig}{id};
									
										$$pContig{$contig_id}{extendclone}{$item}{id}       = $item;
										$$pContig{$contig_id}{extendclone}{$item}{complete} = 1;
										$$pContig{$contig_id}{extendclone}{$item}{orient}   = $orient;
										$$pContig{$contig_id}{extendclone}{$item}{prob}     = $probability;

										$bridge_clones{$item} = 1;						
									}
								}
							}
						}
#------------------------- runs are in different contigs! ----------------------
					} else {
						%reads = ();

						foreach $read ( @reads ){

							$contig_id = $$pRead{$read}{contig}{id};
							
							push @{$reads{$contig_id}{reads}}, $read;
							
							foreach $contig ( keys %reads ){
								@sub_reads = @{$reads{$contig_id}{reads}};

								$distance = 0;
								$orient   = 2;
								
#--------------------- have all reads the same orientation? --------------------
								foreach $read ( @sub_reads ){
									if( $orient != 0 ){
										if( $$pRead{$read}{length} < 0 ){
											if( $orient == -1 || $orient == 2 ){
												$orient = -1;
											} else {
												$orient = 0;
											}
										}
										if( $$pRead{$read}{length} > 0 ){
											if( $orient == 1 || $orient == 2 ){
												$orient = 1;
											} else {
												$orient = 0;
											}
										}
									}
								}
#--------------------- all reads have the same orientation! --------------------
								if( $orient != 0 ){

									if( $orient < 0 ){
										if( $distance < ( $$pRead{$read}{position} + abs( $$pRead{$read}{length} ) - 1 )){
											$distance = $$pRead{$read}{position} + abs( $$pRead{$read}{length} ) - 1;
										}

										if( $distance < $$pBank{$bank}{limit} ){
											$probability = &calculate_likelihood( $distance, $pBank, $bank);

											if( $probability > 0.2 ){

												$contig_id = $$pRead{$read}{contig}{id};
									
												$$pContig{$contig_id}{extendclone}{$item}{id}       = $item;
												$$pContig{$contig_id}{extendclone}{$item}{complete} = 1;
												$$pContig{$contig_id}{extendclone}{$item}{orient}   = $orient;
												$$pContig{$contig_id}{extendclone}{$item}{prob}     = $probability;

												$bridge_clones{$item} = 2;						
											}
										}
									} else {

										foreach $read ( @sub_reads ){

											if( $distance < ( $$pRead{$read}{contig}{length} - $$pRead{$read}{position} + 1 )){
												$distance = $$pRead{$read}{contig}{length} - $$pRead{$read}{position} + 1;
											}
										}
								
										if( $distance < $$pBank{$bank}{limit} ){
											$probability = &calculate_likelihood( $distance, $pBank, $bank);

											if( $probability > 0.2 ){
												$contig_id = $$pRead{$read}{contig}{id};
									
												$$pContig{$contig_id}{extendclone}{$item}{id}       = $item;
												$$pContig{$contig_id}{extendclone}{$item}{complete} = 1;
												$$pContig{$contig_id}{extendclone}{$item}{orient}   = $orient;
												$$pContig{$contig_id}{extendclone}{$item}{prob}     = $probability;

												$bridge_clones{$item} = 2;						
											}
										}
									}
								}
							}
						}
					}

#---------------- there's existing only one read of this clone -----------------
				} else {
					$read = join '.', $item, $$pClone{$item}{runs}[0];
					
					if( $$pRead{$read}{length} < 0 ){
						$distance = $$pRead{$read}{position} + abs( $$pRead{$read}{length} ) - 1;

						if( $distance < $$pBank{$bank}{limit} ){
							$probability = &calculate_likelihood( $distance, $pBank, $bank);

							if( $probability > 0.2 ){

								$contig_id = $$pRead{$read}{contig}{id};						    	   
																								    	   
								$$pContig{$contig_id}{extendclone}{$item}{id}       = $item;	    	   
								$$pContig{$contig_id}{extendclone}{$item}{complete} = 0;		    	   
								$$pContig{$contig_id}{extendclone}{$item}{orient}   = $$pRead{$read}{length}/abs( $$pRead{$read}{length} ); 	   
								$$pContig{$contig_id}{extendclone}{$item}{prob}     = $probability; 	   

								$bridge_clones{$item} = 2;						
							}
						}
					} else {
						$distance = $$pRead{$read}{contig}{length} - $$pRead{$read}{position} + 1;
						
						if( $distance < $$pBank{$bank}{limit} ){
							$probability = &calculate_likelihood( $distance, $pBank, $bank);

							if( $probability > 0.2 ){

								$contig_id = $$pRead{$read}{contig}{id};																       
																																		       
								$$pContig{$contig_id}{extendclone}{$item}{id}       = $item;											       
								$$pContig{$contig_id}{extendclone}{$item}{complete} = 0;												       
								$$pContig{$contig_id}{extendclone}{$item}{orient}   = $$pRead{$read}{length}/abs( $$pRead{$read}{length} );    
								$$pContig{$contig_id}{extendclone}{$item}{prob}     = $probability; 									       

								$bridge_clones{$item} = 2;						
							}
						}
					}
				}
			}
		}
	}
	}
	return( \%bridge_clones );
}





sub calculate_likelihood{
	my( $interval, $pBank_hash, $bank_name ) = @_;					
	my( $i, $tenth, $diff, $low_limit, $high_limit, $likelihood );
	$i = 8;

	while(( $i > 0 ) && ( $$pBank_hash{$bank_name}{dezil}[$i] > $interval )){
		$i--;
	}

	if( $$pBank_hash{$bank_name}{dezil}[$i] > $interval ){
		$low_limit  = 0;
		$high_limit = $$pBank_hash{$bank_name}{dezil}[$i];
		$i--;
	} elsif( $i < 8 ) {
		$low_limit  = $$pBank_hash{$bank_name}{dezil}[$i];
		$high_limit = $$pBank_hash{$bank_name}{dezil}[$i+1];
	} else {
		$low_limit  = $$pBank_hash{$bank_name}{dezil}[$i];
		$high_limit = $$pBank_hash{$bank_name}{limit};
	}

	$tenth      = $high_limit - $low_limit;
	$diff       = $interval - $low_limit;
	$likelihood = 1 - (( $diff / $tenth ) + ( $i + 1 )) / 10;
							
	return( $likelihood );
}

1;
