package QualValues;


use strict;

use vars qw (@EXPORT @ISA);
use Exporter;
@ISA    = qw (Exporter);
@EXPORT = qw ( &coverage &features_exp &exportqualvalues  );


################################################################################
#
# Berechnen der Coverage zu jeder einzelnen Consensus-Base ---> Hash
#
################################################################################


sub coverage{
	my( $pRead, $pContig, $pExternalreads ) = @_;
	
	my( $id, @coverage, $i, $l, $contig );
	
	
# ====================   Fr jeden Contig der Lnge 'l'   ======================
# =========   wird ein Array der Lnge 'l' mit den Werten '0' angelegt   =======

	foreach $id ( keys %$pContig ){
		$l = $$pContig{$id}{length};
		@coverage = ( 0 ) x $l;
		$$pContig{$id}{coverage} = [ @coverage ];
	}
	

# =============   Inkrementiere fr jeden Read (!= Backbone-Read)   ============
# ===============   im entsprechenden Contig-Array die Coverage   ==============

	foreach $id ( keys %$pRead ){

		for( $i = $$pRead{$id}{position}-1; $i < ($$pRead{$id}{position}-1 + abs($$pRead{$id}{length})); $i++){
			$contig = $$pRead{$id}{contig}{id};
			$$pContig{$contig}{coverage}[$i]++ unless( defined $$pExternalreads{$id} );
		}
	}
}


################################################################################
#
# Einfgen von Confidence-Werten, Target-, Backbone-Sequenz in einen Contig-Hash
#
################################################################################


sub features_exp{
	my( $pRead, $pContig, $exp, $name ) = @_;
	
	my( @dummy, @seq, @conf, $seq, $contig, $position, $id, $conf,  );
	
	@dummy = ();
	@seq   = ();
	@conf  = ();
	
	open( EXP, $exp );
	
	while( <EXP> ){
		if ( /^ID   \S+\n/ ){					# ID   NC_001318.1
			( $id ) = m/^ID   (\S+)\n/;			########### DIE CONTIG-ID muss abgefragt werden
		}
		elsif ( /^AV   \s*(.+)\n/ ){			# AV   0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
			( $conf ) = m/^AV   \s*(.+)\n/;
			push( @conf, $conf );
		}
        elsif (m/^   (.+)$/) { # sequences 
          $seq = $1;
          push( @seq, $seq );
		}
		elsif ( /\/\// ){
			$contig   = $$pRead{$id}{contig}{id};
			$position = $$pRead{$id}{position}-1;
			
			$seq = join( " ", @seq );
			$seq =~ s/\s+//g;
            # get unpadded lenght
            my $t_seq = $seq;
            $t_seq =~ s/\*//g;
            $$pContig{$contig}{unpadded_len} = length $t_seq;
            $t_seq = '';
			@seq = split( //, $seq );

			$conf = join( " ", @conf );
			@conf = split( " +", $conf );

			if( $name =~ /target/ ){
				@dummy = ( '*' ) x $#seq;
				$$pContig{$contig}{sequence}   = [ @seq ];
				$$pContig{$contig}{bbsequence} = [ @dummy ];
				$$pContig{$contig}{confidence} = [ @conf ];

			} else {
                print STDERR "\t\tposition:\t$position\t$#{$$pContig{$contig}{bbsequence}}\n\n";
				splice( @{$$pContig{$contig}{bbsequence}}, $position, ($#{$$pContig{$contig}{bbsequence}}+1), @seq );
			}
	
			$conf = "";
			@conf = ();
			$seq  = "";
			@seq  = ();
		}
	}
	
	close( EXP );
}


################################################################################
#
# Ausgabe einer .qual-Tabelle (Basen-Nr. Confidence Coverage)
#
################################################################################


sub exportqualvalues
{  # $pContig = Hash mit -bbseq -confidence -seq(ohne *) -coverage; $filename = , $type = , $rAcontignames = Contig mit Filenames (GAP4 Reihenfolge)
	my( $pContig, $filename, $type, $rAcontignames ) = @_;   

	my( $i, $j, $item, $target );
	open( THECO, ">$filename" );
	
   # type1: backbone (= reference) ohne * , type3: hybrid mit * , type4: target ohne *
	foreach $item (@$rAcontignames)  {
		if ( $type == 3 ){
			printf THECO ">%s\n", $$pContig{$item}{id};

			for( $i = 0; $i < $$pContig{$item}{length}; $i++ ){
				printf THECO "%d\t%d\t%d\n", $i+1, $$pContig{$item}{confidence}[$i], $$pContig{$item}{coverage}[$i];
			}
		} elsif( $type == 1 ){
            if (exists $$pContig{$item}{id}) {
              printf THECO ">%s\n", $$pContig{$item}{id};
            }
            $j = 1;
            my $lenA = scalar (@{$pContig->{$item}->{sequence}});
            for( $i = 0; $i < $lenA; $i++ ) {  
              unless( $$pContig{$item}{sequence}[$i] eq '*') { 
                printf THECO "%d\t%d\t%d\n", $j, $$pContig{$item}{confidence}[$i], $$pContig{$item}{coverage}[$i];
                $j++;
              }
            }
        } elsif( $type == 4 ){
			$target = 0;
			$j      = 1;

			for( $i = 0; $i < $$pContig{$item}{length}; $i++ ){

				if( $$pContig{$item}{coverage}[$i] > 0 ){

					if( $target == 0 ){
						print THECO ">$$pContig{$item}{$i+1}{read}[0]\n";
					}
					if( $$pContig{$item}{sequence}[$i] !~ /\*/ ){
						printf THECO "%d\t%d\t%d\n", $j, $$pContig{$item}{confidence}[$i], $$pContig{$item}{coverage}[$i];
					$j++;
					}
					$target = 1;
				} else {
					$j      = 1;
					$target = 0;
				}
			}
      }
      elsif ($type eq 'project_unpadded')
      {  # mbs
         my @print_A;
         my $txt = sprintf ">%s\n", $$pContig{$item}{id};
         push(@print_A, $txt);
         
         my $out_pos_num = 1;
         
         # print value lines 
         for( my $base_num = 0; $base_num < $$pContig{$item}{unpadded_len}; $base_num++ )
         {
            unless( $$pContig{$item}{sequence}[$base_num] eq '*')
            {
               $txt = sprintf "%d\t%d\t%d\n", $out_pos_num, $$pContig{$item}{confidence}[$base_num], $$pContig{$item}{coverage}[$base_num];
               push(@print_A, $txt);
               $out_pos_num++;
            }
         }
         
         # add num of values to header
         chomp $print_A[0];
         $print_A[0] = $print_A[0] . ' ' . ($out_pos_num-1) . " values \n";
         
         # print contig values
         print THECO @print_A;
         @print_A = ();
		}
      else {
			print STDERR "\t\tMissing Backbone Name\n\n";
			print STDERR "\t\tTry It Again!\n\n";
			exit 0;
		}
	}
	close( THECO );
}


1;
