################################################################################
#
#  all rights reserved
#  Karol Szafranski at IMB Jena, Dept. Genome Analysis, 2001-2002,2004
#  szafrans@imb-jena.de
#
################################################################################

package Range;

# includes
use strict;
use DefaultObjArray; 

# inheritance
our @ISA;
push @ISA, qw(DefaultObjArray);

# overloading
use overload
'""'   => "_Stringify",
'bool' => sub { 1; },
'%{}'  => sub { return { '-1'=>$_[0][0], '1'=>$_[0][1] }; },
;

# package-wide constants and variables
our (%LibGlob);



# shouldn't we use %LibGlob instead?
$LibGlob{switch} = {};


# parametric initialisation
#
# INTERFACE
# - argument 1+: either:
#                - with 1 arg:
#                  - class::$arg1
#                  - $arg1 = [$arg1a,$arg1b]
#                - with 2 args: $arg1..$arg2
#
# - library options:
#   -debug      STD
#
# - return val: - object reference
#               - undef if an error occurs
#
# DESCRIPTION
# - this function will be called by method new()
# - either range argument may be undef => 'border not defined'
#
sub ini {
  my ($this, @DataArg) = @_;
  my ($class, $debug);

  # function parameters
  $class = ref ($this);
  $debug = $LibGlob{switch}{-debug};

  # enter submitted argument(s)
  if (@DataArg) {

    # convert refs to array:
    # this object, ARRAY reference
    if (@DataArg == 1) {
      if (ref($DataArg[0]) eq 'ARRAY' and int(@{$DataArg[0]})==2) {
        @DataArg = @{$DataArg[0]};
      } elsif (ref($DataArg[0]) eq $class) {
        @DataArg = @{$DataArg[0]};
      } else {
        printf STDERR "%s. ERROR: unable to interpet data argument %s\n", (caller(0))[3], $DataArg[0]||"''";
        return undef;
      }
    }

    # assign from array of points
    if (@DataArg == 2) {
      $debug and printf STDERR "%s. assigning from arguments: (%s..%s)\n", (caller(0))[3], @DataArg;
      if (! int (grep { ! defined $_ } @DataArg) and $DataArg[0] > $DataArg[1]) {
        printf STDERR "%s. ERROR: conflicting arguments: (%s..%s)\n", (caller(0))[3], @DataArg;
        return undef;
      } else {
        @$this = @DataArg;
      }
    }

    # arg number error
    else {
      printf STDERR "%s. ERROR: too many data arguments %d\n", (caller(0))[3], int @DataArg;
      return undef;
    }
  }

  # no args
  else {
    printf STDERR "%s. ERROR: cannot do initialisation without args\n", (caller(0))[3];
    return undef;
  }

  return $this;
}



# return class instance
#
# INTERFACE
# - argument 1*: initialisation arguments
# - return val:  reference on object
#
# DESCRIPTION
# - see class function for detailed description of initialisation parameter
#   convention.
# - this function will call method ini() in the derived class
sub new {
  my $this = [];
  my ($class, @arg) = @_;

  bless $this, ref($class) || $class;
  return $this->ini(@arg);
}



# parse object from string argument
#
# INTERFACE
# - argument 1: string range argument
# - return val: object reference
#
sub Parse {
  my ($this, $ArgRange) = @_;
  my (@range);

  # refine argument string
  $ArgRange =~ s/\s+//g;

  # default for minimal argument
  # - left range only
  if ($ArgRange !~ m/\.\./) {
    $ArgRange .= '..';
  }

  # format
  @range = (split /\.\./, $ArgRange)[0..1];
  @range = map { ($_ eq '') ? undef : $_; } @range;

  return $this->new(\@range);
}



# set or return left border of range
#
sub Left {
  my ($this, $val) = @_;

  if (defined ($val)) {
    $this->[0] = $val;
  }
  
  # return actual value
  return $this->[0];
}


# set or return right border of range
#
sub Right {
  my ($this, $val) = @_;

  if (defined ($val)) {
    $this->[1] = $val;
  }
  
  # return actual value
  return $this->[1];
}



# string representation object
#
# INTERFACE
# - return val: stringified object
#
sub _Stringify {
  my ($this) = @_;
  my ($RangePln);

  # format
  $RangePln = sprintf ("%s(%s..%s)",
    ref($this), 
    map { defined($_)?"$_":'' } @$this,
    );

  return $RangePln;
}


1;
