################################################################################
#
#  all rights reserved
#  Karol Szafranski at IMB Jena, Dept. Genome Analysis, 2001-2002,2004
#  szafrans@imb-jena.de
#
################################################################################

package RangeGapped;

# includes
#use strict;

# symbol export
use vars qw (@ISA @EXPORT);
use Exporter;
@ISA    = qw (Exporter);
@EXPORT = qw (
  %SeqSmb 
  &SeqRangeGapped
  );


# sequence symbols
#
use vars qw (%SeqSmb);
%SeqSmb = (
  'all'       => 'ABCDEFGHIKLMNPQRSTUVWXY',
  'DNA'       => 'ACGT',
  'DNA-unk'   => 'ACGTN',
  'RNA'       => 'ACGU',
  'nt'        => 'ACGTU',
  'nt-unk'    => 'ACGTUN',
  'nt-wobble' => 'ACGTUBDHKMNRSVWY',
  'prot'      => 'ACDEFGHIKLMNPQRSTVWY',
  'prot-unk'  => 'ACDEFGHIKLMNPQRSTVWYX',
  );

# map range data from gap-free sequence string to gapped sequence string
#
# INTERFACE
# - argument 1: sequence string purified
# - argument 2: sequence string gapped
# - argument 3: reference to range object (or reference to data structure
#               which may be used to create a range object) referring to.
#               the ungapped sequence.
#
# - options:
#   -debug      [STD]
#
# - return val: - reference to range object which is equivalent to
#                 hash ref { '-1'=>$PosLeft, '1'=>$PosRight }
#               - undef if an error occurred
#
# DESCRIPTION
# - the function selects the minimal range on the gapped sequence
#   i.e. the range won't contain gaps adjacent to the borders.
#
sub SeqRangeGapped {
  my ($SeqPure, $SeqOrig, $pRange, %opt) = @_;
  my ($debug, $CharCode);
  my $CtGap=0;

  # function parameters
  $debug = $opt{-debug};
  $CharCode = lc ($SeqSmb{all}) . uc ($SeqSmb{all});
  #$pRange = Math::Range->new ($pRange) or return undef; #mf
  $pRange = Range->new ($pRange) or return undef; #mf

  # anything to do here?
  if ($SeqOrig !~ m/[^$CharCode]+/o) {
    return $pRange;
  }

  # scan for gaps
  while ($SeqOrig =~ m/[^$CharCode]/og and
         pos ($SeqOrig) <= ($pRange->Left()+$CtGap)) {
    $CtGap ++;
  }
  $pRange->Left ($pRange->Left() + $CtGap);
  pos ($SeqOrig) = $pRange->Left();

  # scan for gaps
  while ($SeqOrig =~ m/[^$CharCode]/og and
         pos ($SeqOrig) <= ($pRange->Right()+$CtGap)) {
    $CtGap ++;
  }
  $pRange->Right ($pRange->Right() + $CtGap);

  # exit SUB
  return $pRange;
}


1;
